///////////////////////////////////////////////////////////////////////////////
//                           Function defenitions                            //
///////////////////////////////////////////////////////////////////////////////

// These actions control the tutorials in which the player has no interaction with.
Trigger.prototype.WelcomeMessage = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "Welcome to the basic tutorial of the 0 A.D.!");
	this.DoAfterDelay(3000, "IntroductionMessage", {});
};

Trigger.prototype.IntroductionMessage = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "In this tutorial you will learn the basics of the gameplay of 0 A.D.");
	this.DoAfterDelay(6000, "CameraMovementTutorial1", {});
};

Trigger.prototype.CameraMovementTutorial1 = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "First, we try the camera movements. To move your camera, move the mouse pointer to the edge of the screen.");
	this.DoAfterDelay(8000, "CameraMovementTutorial2", {});
};

Trigger.prototype.CameraMovementTutorial2 = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "You can also move the camera using W,A,S,D or arrow keys on the keyboard, or by holding the middle mouse button and moving it.");
	this.DoAfterDelay(15000, "CameraMovementTutorial3", {});
};

Trigger.prototype.CameraMovementTutorial3 = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "To rotate the camera, hold Ctrl and one of the arrow keys.");
	this.DoAfterDelay(7000, "CameraMovementTutorial4", {});
};

Trigger.prototype.CameraMovementTutorial4 = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "To zoom, use the mouse scroll or + and - keys on the keyboard.");
	this.DoAfterDelay(7000, "IntroductionToUnits", {});
};

Trigger.prototype.IntroductionToUnits = function(data)
{
	if (this.state != "start")
		return;
	TriggerHelper.PushGUINotification(1, "Now it is time to work with units. Move the mouse pointer on one of the units present in the screen, and left-click to select it. Then, right-click on an open part of the terrain.");
	this.state = "movingTutorial1";
	this.EnableTrigger("OnInterval", "MovingUnitsTutorial");
	this.EnableTrigger("OnPlayerCommand", "HandleOrders");
};

Trigger.prototype.IntroductionToResources = function(data)
{
	if (this.state != "gatheringFruit")
		return;
	TriggerHelper.PushGUINotification(1, "Resource gathering plays a central role in 0 A.D.. There are four resources found in the game. Food, wood, stone and metal. You can find your current stockpile in the upper left section of the screen.");
	this.DoAfterDelay(15000, "MoreDescriptionAboutResources", {});
};

Trigger.prototype.MoreDescriptionAboutResources = function(data)
{
	if (this.state != "gatheringFruit")
		return;
	TriggerHelper.PushGUINotification(1, "Resources are required to train units, build structures and research technologies.");
	this.EnableTrigger("OnInterval", "TutorialOnGatheringFood");
};

// These actions remind the players of what they should do.
Trigger.prototype.MovingUnitsTutorial = function(data)
{
	if (this.state != "movingTutorial1")
		return;
	TriggerHelper.PushGUINotification(1, "Select a unit by left clicking on it, and then, right-click on an open part of the terrain.");
};

Trigger.prototype.MovingAGroupOfUnitsTutorial = function(data)
{
	if (this.state != "movingTutorial2")
		return;
	TriggerHelper.PushGUINotification(1, "Hold the left button of the mouse and drag it to select a group of units. Then, order them to move.");
};

Trigger.prototype.TutorialOnGatheringFood = function(data)
{
	if (this.state != "gatheringFruit")
		return;
	TriggerHelper.PushGUINotification(1, "Food is gathered from farms, fruit trees and bushes, animals, fishes and farms. Select some of your units and right-click on the berry bushes in the east to gather food.");
};

Trigger.prototype.TutorialOnGatheringWood = function(data)
{
	if (this.state != "gatheringWood")
		return;
	TriggerHelper.PushGUINotification(1, "Wood is gathered from trees. Select some of your units and right-click on a tree to gather wood.");
};

Trigger.prototype.TutorialOnGatheringStone = function(data)
{
	if (this.state != "gatheringStone")
		return;
	TriggerHelper.PushGUINotification(1, "Stone is gathered from stone quarries. Select some of your units and right-click on the quarry in the south of your town to gather stone.");
};

Trigger.prototype.TutorialOnGatheringMetal = function(data)
{
	if (this.state != "gatheringMetal")
		return;
	TriggerHelper.PushGUINotification(1, "Metal is gathered from metal ores. Select some of your units and right-click on the rock in the north-east of your town to gather metal.");
};

Trigger.prototype.TutorialOnExploration = function(data)
{
	if (this.state != "goToTriggerPoint")
		return;
	TriggerHelper.PushGUINotification(1, "Notice that you can only see the immidiate surroundings of you town. To see further, select your rider and order him to move along the south-eastern road into the black area.");
};

Trigger.prototype.TutorialOnGatheringTreasure = function(data)
{
	if (this.state != "gatheringTreasure")
		return;
	TriggerHelper.PushGUINotification(1, "Order your rider to gather the treasures. To do so, select your rider and right-click on one of the treasures near the road.");
};

// Thse actions set the next stage of the tutorial after the player has successfully done the previous ones

Trigger.prototype.HandleOrders = function(data)
{
	if (data.cmd.type == "walk")
	{
		if (data.cmd.entities.length == 1)
			this.PlayerMovedAUnit(data);
		else if (data.cmd.entities.length > 1)
			this.PlayerMovedMultipleUnits(data);
	}
	else if (data.cmd.type == "gather")
	{
		var type = TriggerHelper.GetResourceType(data.cmd.target);
		if (type === undefined)
			return;
		else if (type.specific == "fruit")
			this.UnitOrderedToGatherFruit(data);
		else if (type.generic == "wood")
			this.UnitOrderedToGatherWood(data);
		else if (type.generic == "stone")
			this.UnitOrderedToGatherStone(data);
		else if (type.generic == "metal")
			this.UnitOrderedToGatherMetal(data);
		else if (type.generic == "treasure")
			this.UnitOrderedToGatherTreasure(data);
	}
};

// This one is called when the player moves a unit.
Trigger.prototype.PlayerMovedAUnit = function(data)
{
	if (this.state != "movingTutorial1")
		return;
	this.state = "movingTutorial2";
	this.DisableTrigger("OnInterval", "MovingUnitsTutorial");
	TriggerHelper.PushGUINotification(1, "Good. You can also select a group of units by holding the left mouse button and dragging it. Select a group of units and order them to move.");
	this.EnableTrigger("OnInterval", "MovingAGroupOfUnitsTutorial");
};

Trigger.prototype.PlayerMovedMultipleUnits = function(data)
{
	if (this.state != "movingTutorial2")
		return;
	this.state = "gatheringFruit";
	this.DisableTrigger("OnInterval", "MovingAGroupOfUnitsTutorial");
	TriggerHelper.PushGUINotification(1, "Very Good.");
	this.DoAfterDelay(6000, "IntroductionToResources", {});
};

// This one is called when the player orders a unit to gather from fruit bushes
Trigger.prototype.UnitOrderedToGatherFruit = function(data)
{
	if (this.state != "gatheringFruit")
		return;
	this.state = "gatheringWood";
	TriggerHelper.PushGUINotification(1, "Excelent. Now your units will move and gather food from those bushes.");
	this.DisableTrigger("OnInterval", "TutorialOnGatheringFood");
	this.EnableTrigger("OnInterval", "TutorialOnGatheringWood");
};

// This one is called when the player orders a unit to gather from trees
Trigger.prototype.UnitOrderedToGatherWood = function(data)
{
	if (this.state != "gatheringWood")
		return;
	this.state = "gatheringStone";
	TriggerHelper.PushGUINotification(1, "Perfect. Now your units will move and gather wood from the trees.");
	this.DisableTrigger("OnInterval", "TutorialOnGatheringWood");
	this.EnableTrigger("OnInterval", "TutorialOnGatheringStone");
};

// This one is called when the player orders a unit to gather from stone quarries
Trigger.prototype.UnitOrderedToGatherStone = function(data)
{
	if (this.state != "gatheringStone")
		return;
	this.state = "gatheringMetal";
	TriggerHelper.PushGUINotification(1, "Good. Now your units will move and gather stone from the quarry.");
	this.DisableTrigger("OnInterval", "TutorialOnGatheringStone");
	this.EnableTrigger("OnInterval", "TutorialOnGatheringMetal");
};

// This one is called when the player orders a unit to gather from metal mines
Trigger.prototype.UnitOrderedToGatherMetal = function(data)
{
	if (this.state != "gatheringMetal")
		return;
	this.state = "goToTriggerPoint";
	TriggerHelper.PushGUINotification(1, "Very Good. Now your units will move and gather metal from the ore.");
	this.DisableTrigger("OnInterval", "TutorialOnGatheringMetal");

	// Spawn a cavalry unit from trigger point B (near the player's CC)
	TriggerHelper.SpawnUnitsFromTriggerPoints("B", "skirmish/units/default_cavalry", 1, 1)
	this.EnableTrigger("OnInterval", "TutorialOnExploration");
	this.EnableTrigger("OnRange", "StartTreasureTutorial");
};

// This one is called when a unit gets near the trigger entity in the south-east
Trigger.prototype.StartTreasureTutorial = function(data)
{
	if (this.state != "goToTriggerPoint")
		return;
	this.state = "gatheringTreasure";
	TriggerHelper.PushGUINotification(1, "You have found some treasures. Treasures give large ammount of their representing resources instantly. Order your rider to gather the treasures.");
	this.DisableTrigger("OnInterval", "TutorialOnExploration");
	this.DisableTrigger("OnRange", "StartTreasureTutorial");
	this.EnableTrigger("OnInterval", "TutorialOnGatheringTreasure");
};

// This one is called when the player orders a unit to gather from treasures
Trigger.prototype.UnitOrderedToGatherTreasure = function(data)
{
	if (this.state != "gatheringTreasure")
		return;
	this.state = "complete";
	TriggerHelper.PushGUINotification(1, "Outstanding. This tutorial is now done.");
	this.DisableTrigger("OnInterval", "TutorialOnGatheringTreasure");
	this. gatheringTreasureCompleted = true;
	this.DoAfterDelay(4000, "Victory", {});
};

Trigger.prototype.Victory = function(data)
{
	TriggerHelper.SetPlayerWon(1);
};

///////////////////////////////////////////////////////////////////////////////
//                            Startup Actions                                //
///////////////////////////////////////////////////////////////////////////////

// This function is run in the start of the game. We should initialize everything here.
Trigger.prototype.InitGame = function(data)
{
	// Our trigger variables should be defined here or they will be lost
	// after a save.
	// This map will use some states, to know in which part of the tutorial it is
	// possible states:
	//[ start, movingTutorial1, movingTutorial2, gatheringFruit, gatheringWood, gatheringStone, gatheringMetal, goToTriggerPoint,gatheringTreasure, complete]
	this.state = "start";

	// Start the first tutorial after 2 seconds.
	this.DoAfterDelay(2000, "WelcomeMessage", {});

	///////////////////////////////////////////////////////////////////////////
	//                           Register Triggers                           //
	///////////////////////////////////////////////////////////////////////////

	// For a trigger to work, actions should be bound to an event.
	var timerData = {"enabled": false, "delay": 10000, "interval": 12000}
	this.RegisterTrigger("OnInterval", "MovingUnitsTutorial", timerData);
	timerData.interval = 10000;
	this.RegisterTrigger("OnInterval", "MovingAGroupOfUnitsTutorial", timerData);
	timerData.delay = 5000
	this.RegisterTrigger("OnInterval", "TutorialOnGatheringFood", timerData);
	this.RegisterTrigger("OnInterval", "TutorialOnGatheringWood", timerData);
	this.RegisterTrigger("OnInterval", "TutorialOnGatheringStone", timerData);
	this.RegisterTrigger("OnInterval", "TutorialOnGatheringMetal", timerData);
	this.RegisterTrigger("OnInterval", "TutorialOnExploration", timerData);
	this.RegisterTrigger("OnInterval", "TutorialOnGatheringTreasure", timerData);

	// the query data to be used for the following range query
	var queryData = {
		"entities": this.GetTriggerPoints("A"), // source of the range trigger (the trigger entity A on this map)
		"maxRange": 60, // maximum range
		"players": [1], // only accept results from player 1
		"requiredComponent": IID_UnitAI, // require the results to implement the UnitAI
		"enabled": false, // disabled for now
	}
	// Range triggers are run when a unit enters or leaves the circle around the source
	this.RegisterTrigger("OnRange", "StartTreasureTutorial", queryData);
	
	
	// Normal triggers. The first parameter is the "event". The second one is the action
	// By default, the trigger is disabled
	this.RegisterTrigger("OnPlayerCommand", "HandleOrders");
};

