var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);

// define some globally accessible variables
g_Triggers.movingTutorial1Completed = false;
g_Triggers.movingTutorial2Completed = false;
g_Triggers.gatheringFruitComepleted = false;
g_Triggers.gatheringTreeComepleted = false;
g_Triggers.gatheringStoneComepleted = false;
g_Triggers.gatheringMetalComepleted = false;
g_Triggers.gatheringTreasureComepleted = false;
g_Triggers.exploredTerrain = false;
g_Triggers.movingTutorial2EntitiesOrdered = 0;

////////////////////////////////////////////////////////////////////////////////
//                                Function defenitions                        //
////////////////////////////////////////////////////////////////////////////////

// Thse actions set the next stage of the tutorial after the player has successfully done the previous ones
g_Triggers.HandleOrders = function(data)
{
	if (data.order == "Order.Walk" || data.order == "Order.FormationWalk")
		g_Triggers.PlayerMovedAUnit(data);
	else if (data.order == "Order.Gather")
	{
		var type = GetEntityResourceType(data.target);
		if (!type)
			return;
		else if (type.specific == "fruit")
			g_Triggers.UnitOrderedToGatherFruit(data);
		else if (type.specific == "tree")
			g_Triggers.UnitOrderedToGatherTree(data);
		else if (type.specific == "rock")
			g_Triggers.UnitOrderedToGatherStoneQuarry(data);
		else if (type.specific == "ore")
			g_Triggers.UnitOrderedToGatherMetalMine(data);
		else if (type.generic == "treasure")
			g_Triggers.UnitOrderedToGatherTreasure(data);
	}
}
cmpTrigger.RegisterTrigger("OnUnitIssuedOrder", "HandleOrders");

// These actions control the tutorials in which the player has no interactin with.
cmpTrigger.DoAfterDelay(1000, "WelcomeMessage", {});
g_Triggers.WelcomeMessage = function(data)
{
	PushNotification(1, "Welcome to the basic tutorial of the 0 A.D.!");
	cmpTrigger.DoAfterDelay(3000, "IntroductionMessage", {});
}

g_Triggers.IntroductionMessage = function(data)
{
	PushNotification([1], "In this tutorial you will learn the basics of the gameplay of 0 A.D.");
	cmpTrigger.DoAfterDelay(6000, "CameraMovementTutorial1", {});
}

g_Triggers.CameraMovementTutorial1 = function(data)
{
	PushNotification([1], "First, we try the camera movements. To move your camera, move the mouse pointer to the edge of the screen.");
	cmpTrigger.DoAfterDelay(8000, "CameraMovementTutorial2", {});
}

g_Triggers.CameraMovementTutorial2 = function(data)
{
	PushNotification([1], "You can also move the camera using W,A,S,D or arrow keys on the keyboard, or by holding the middle mouse button and moving it.");
	cmpTrigger.DoAfterDelay(12000, "CameraMovementTutorial3", {});
}

g_Triggers.CameraMovementTutorial3 = function(data)
{
	PushNotification([1], "To rotate the camera, hold Ctrl and one of the arrow keys.");
	cmpTrigger.DoAfterDelay(7000, "IntroductionToUnits", {});
}

// MOVING
// This one is called when the player moves a unit.
g_Triggers.PlayerMovedAUnit = function(data)
{
	if (!g_Triggers.movingTutorial1Completed)
	{
		g_Triggers.movingTutorial1Completed = true;
		PushNotification([1], "Good. You can also select a group of units by holding the left mouse button and dragging it. Select a group of units and order them to move.");
		cmpTrigger.DoAfterDelay(12000, "MovingAGroupOfUnitsTutorial");
	}
	else if (!g_Triggers.movingTutorial2Completed)
	{
		++g_Triggers.movingTutorial2EntitiesOrdered;
		if (g_Triggers.movingTutorial2EntitiesOrdered >= 2)
		{
			g_Triggers.movingTutorial2Completed = true;
			PushNotification([1], "Very Good.");
			cmpTrigger.DoAfterDelay(6000, "IntroductionToResources", {});
		}
	}
}

g_Triggers.IntroductionToUnits = function(data)
{
	PushNotification([1], "Now it is time to work with units. Move the mouse pointer on one of the units present in the screen, and left-click to select it. Then, right-click on an open part of the terrain.");
	cmpTrigger.DoAfterDelay(12000, "MovingUnitsTutorial"); 
}

g_Triggers.MovingUnitsTutorial = function(data)
{
	if (g_Triggers.movingTutorial1Completed)
		return;
	PushNotification([1], "Select a unit by left clicking on it, and then, right-click on an open part of the terrain.");
	cmpTrigger.DoAfterDelay(12000, "MovingUnitsTutorial"); 
}

g_Triggers.MovingAGroupOfUnitsTutorial = function(data)
{
	if (g_Triggers.movingTutorial2EntitiesOrdered >= 2)
		return;
	PushNotification([1], "Hold the left button of the mouse and drag it to select a group of units. Then, order them to move.");
	cmpTrigger.DoAfterDelay(12000, "MovingAGroupOfUnitsTutorial"); 
}

// RESOURCES
g_Triggers.IntroductionToResources = function(data)
{
	PushNotification([1], "Resource gathering play a central role in 0 A.D.. There are four resources found in the game. Food, Wood, Stone and metal. You can find your current stockpile in the upper left section of the screen.");
	cmpTrigger.DoAfterDelay(12000, "MoreDescriptionAboutResources", {});
}

g_Triggers.MoreDescriptionAboutResources = function(data)
{
	PushNotification([1], "Resources are required to train units, build structures and research technologies.");
	cmpTrigger.DoAfterDelay(4000, "TutorialOnGatheringFood", {});
}

g_Triggers.TutorialOnGatheringFood = function(data)
{
	if (g_Triggers.gatheringFruitComepleted)
		return;
	PushNotification([1], "Food is gathered from farms, fruit trees and bushes, animals, fishes and farms. Select some of your units and right-click on the berry bushes in the east to gather food.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnGatheringFood", {});
}

g_Triggers.UnitOrderedToGatherFruit = function(data)
{
	if (g_Triggers.gatheringFruitComepleted)
		return;
	PushNotification([1], "Excelent. Now your units will move and gather food from those bushes.");
	
	cmpTrigger.DoAfterDelay(2000, "TutorialOnGatheringWood");
	g_Triggers.gatheringFruitComepleted = true;
}

g_Triggers.TutorialOnGatheringWood = function(data)
{
	if (g_Triggers.gatheringTreeComepleted)
		return;
	PushNotification([1], "Wood is gathered from trees. Select some of your units and right-click on a tree to gather wood.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnGatheringWood", {});
}

g_Triggers.UnitOrderedToGatherTree = function(data)
{
	if (g_Triggers.gatheringTreeComepleted)
		return;
	PushNotification([1], "Perfect. Now your units will move and gather wood from the trees.");
	cmpTrigger.DoAfterDelay(2000, "TutorialOnGatheringStone");
	g_Triggers.gatheringTreeComepleted = true;
}

g_Triggers.TutorialOnGatheringStone = function(data)
{
	if (g_Triggers.gatheringStoneComepleted)
		return;
	PushNotification([1], "Stone is gathered from stone quarries. Select some of your units and right-click on the quarry in the south of your town to gather stone.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnGatheringStone", {});
}

// This one is called when the player orders a unit to gather from stone quarries
g_Triggers.UnitOrderedToGatherStoneQuarry = function(data)
{
	if (g_Triggers.gatheringStoneComepleted)
		return;
	PushNotification([1], "Good. Now your units will move and gather stone from the quarry.");
	cmpTrigger.DoAfterDelay(2000, "TutorialOnGatheringMetal");
	g_Triggers.gatheringStoneComepleted = true;
}

g_Triggers.TutorialOnGatheringMetal = function(data)
{
	if (g_Triggers.gatheringMetalComepleted)
		return;
	PushNotification([1], "Metal is gathered from metal ores. Select some of your units and right-click on the rock in the north-east of your town to gather metal.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnGatheringMetal", {});
}

// This one is called when the player orders a unit to gather from metal mines
g_Triggers.UnitOrderedToGatherMetalMine = function(data)
{
	if (g_Triggers.gatheringMetalComepleted)
		return;
	PushNotification([1], "Very Good. Now your units will move and gather metal from the ore.");
	
	// Spawn a cavalry unit for the player in his/her civic center.
	var playerEntities = GetEntitiesByPlayer(1); // Get all of the player's entities
	for each (entity in playerEntities)
	{
		if (EntityHasClass(entity, "CivilCentre"))
		{
			BuildingSpawnUnits(entity, "skirmish/units/default_cavalry", 1);
			break;
		}
	}
	
	cmpTrigger.DoAfterDelay(2000, "TutorialOnExploration");
	cmpTrigger.RegisterOnUnitRangeFromEntityTrigger(226, 50, "StartTreasureTutorial", [1], IID_DamageReceiver); // This special trigger calls the action when an entity owned by player 1 enters a circular area of 50 game meters centered around entity 226 (the gaia market).
	
	g_Triggers.gatheringMetalComepleted = true;
}

g_Triggers.TutorialOnExploration = function(data)
{
	if (g_Triggers.exploredTerrain)
		return;
	PushNotification([1], "Notice that you can only see the immidiate surroundings of you town. To see further, select your rider and order him to move along the south-eastern road into the black area.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnExploration");
}

// This one is called when a unit gets near the market in the south-east
g_Triggers.StartTreasureTutorial = function(data)
{
	if (!data.added.length || g_Triggers.exploredTerrain)
		return;
	PushNotification([1], "You have found some treasures. Treasures give large ammount of their representing resources instantly. Order your rider to gather the treasures.");
	g_Triggers.exploredTerrain = true;
	cmpTrigger.DisableRangeTrigger("StartTreasureTutorial");
	cmpTrigger.DoAfterDelay(2000, "TutorialOnGatheringTreasure");
}

g_Triggers.TutorialOnGatheringTreasure = function(data)
{
	if (g_Triggers.gatheringTreasureComepleted)
		return;
	PushNotification([1], "Order your rider to gather the treasures. To do so, select your rider and right-click on one of the treasures near the road.");
	cmpTrigger.DoAfterDelay(12000, "TutorialOnGatheringTreasure");
}

// This one is called when the player orders a unit to gather from treasures
g_Triggers.UnitOrderedToGatherTreasure = function(data)
{
	if (g_Triggers.gatheringTreasureComepleted)
		return;
	PushNotification([1], "Outstanding. This tutorial is now done.");
	g_Triggers.gatheringTreasureComepleted = true;
	
	cmpTrigger.DoAfterDelay(2000, "Victory", {});
}

g_Triggers.Victory = function(data)
{
	DefeatPlayer(2);
}


