// Global variables
var g_TriggerData = {};

g_TriggerData.cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);

g_TriggerData.movingTutorial1Completed = false;
g_TriggerData.movingTutorial2Completed = false;
g_TriggerData.gatheringFruitComepleted = false;
g_TriggerData.gatheringTreeComepleted = false;
g_TriggerData.gatheringStoneComepleted = false;
g_TriggerData.gatheringMetalComepleted = false;
g_TriggerData.gatheringTreasureComepleted = false;


///////////////////////////////////////////////////////////////////////////////
//                           Function defenitions                            //
///////////////////////////////////////////////////////////////////////////////

// These actions control the tutorials in which the player has no interaction with.
g_TriggerData.action_WelcomeMessage = function(data)
{
	PushGUINotificationInTopCenter(1, "Welcome to the basic tutorial of the 0 A.D.!");
	g_TriggerData.cmpTrigger.DoAfterDelay(3000, "action_IntroductionMessage", {});
}

g_TriggerData.action_IntroductionMessage = function(data)
{
	PushGUINotificationInTopCenter(1, "In this tutorial you will learn the basics of the gameplay of 0 A.D.");
	g_TriggerData.cmpTrigger.DoAfterDelay(6000, "action_CameraMovementTutorial1", {});
}

g_TriggerData.action_CameraMovementTutorial1 = function(data)
{
	PushGUINotificationInTopCenter(1, "First, we try the camera movements. To move your camera, move the mouse pointer to the edge of the screen.");
	g_TriggerData.cmpTrigger.DoAfterDelay(8000, "action_CameraMovementTutorial2", {});
}

g_TriggerData.action_CameraMovementTutorial2 = function(data)
{
	PushGUINotificationInTopCenter(1, "You can also move the camera using W,A,S,D or arrow keys on the keyboard, or by holding the middle mouse button and moving it.");
	g_TriggerData.cmpTrigger.DoAfterDelay(15000, "action_CameraMovementTutorial3", {});
}

g_TriggerData.action_CameraMovementTutorial3 = function(data)
{
	PushGUINotificationInTopCenter(1, "To rotate the camera, hold Ctrl and one of the arrow keys.");
	g_TriggerData.cmpTrigger.DoAfterDelay(7000, "action_IntroductionToUnits", {});
}

g_TriggerData.action_IntroductionToUnits = function(data)
{
	PushGUINotificationInTopCenter(1, "Now it is time to work with units. Move the mouse pointer on one of the units present in the screen, and left-click to select it. Then, right-click on an open part of the terrain.");
	g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_MovingUnitsTutorial");
	g_TriggerData.cmpTrigger.StartUpdateTimer(10000, 12000); // Sets the interval in which the event "Always" occurs to 12000ms (12 seconds). Used to remind the players of what they should do.
}

g_TriggerData.action_IntroductionToResources = function(data)
{
	PushGUINotificationInTopCenter(1, "Resource gathering play a central role in 0 A.D.. There are four resources found in the game. Food, Wood, Stone and metal. You can find your current stockpile in the upper left section of the screen.");
	g_TriggerData.cmpTrigger.DoAfterDelay(15000, "action_MoreDescriptionAboutResources", {});
}

g_TriggerData.action_MoreDescriptionAboutResources = function(data)
{
	PushGUINotificationInTopCenter(1, "Resources are required to train units, build structures and research technologies.");
	g_TriggerData.cmpTrigger.DoAfterDelay(4000, "action_EnablingGatheringFoodTrigger", {});
}

g_TriggerData.action_EnablingGatheringFoodTrigger = function(data)
{
	g_TriggerData.cmpTrigger.StartUpdateTimer(1000, 10000); // Sets the interval in which the event "Always" occurs to 10000ms (10 seconds). Used to remind the players of what they should do.
	g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnGatheringFood");
}

// These actions remind the players of what they should do.
g_TriggerData.action_MovingUnitsTutorial = function(data)
{
	PushGUINotificationInTopCenter(1, "Select a unit by left clicking on it, and then, right-click on an open part of the terrain.");
}

g_TriggerData.action_MovingAGroupOfUnitsTutorial = function(data)
{
	PushGUINotificationInTopCenter(1, "Hold the left button of the mouse and drag it to select a group of units. Then, order them to move.");
}

g_TriggerData.action_TutorialOnGatheringFood = function(data)
{
	PushGUINotificationInTopCenter(1, "Food is gathered from farms, fruit trees and bushes, animals, fishes and farms. Select some of your units and right-click on the berry bushes in the east to gather food.");
}

g_TriggerData.action_TutorialOnGatheringWood = function(data)
{
	PushGUINotificationInTopCenter(1, "Wood is gathered from trees. Select some of your units and right-click on a tree to gather wood.");
}

g_TriggerData.action_TutorialOnGatheringStone = function(data)
{
	PushGUINotificationInTopCenter(1, "Stone is gathered from stone quarries. Select some of your units and right-click on the quarry in the south of your town to gather stone.");
}

g_TriggerData.action_TutorialOnGatheringMetal = function(data)
{
	PushGUINotificationInTopCenter(1, "Metal is gathered from metal ores. Select some of your units and right-click on the rock in the north-east of your town to gather metal.");
}

g_TriggerData.action_TutorialOnExploration = function(data)
{
	PushGUINotificationInTopCenter(1, "Notice that you can only see the immidiate surroundings of you town. To see further, select your rider and order him to move along the south-eastern road into the black area.");
}

g_TriggerData.action_TutorialOnGatheringTreasure = function(data)
{
	PushGUINotificationInTopCenter(1, "Order your rider to gather the treasures. To do so, select your rider and right-click on one of the treasures near the road.");
}

// Thse actions set the next stage of the tutorial after the player has successfully done the previous ones

g_TriggerData.action_HandleOrders = function(data)
{
	if (data.order == "Order.Walk")
		g_TriggerData.action_PlayerMovedAUnit(data);
	else if (data.order == "Order.Gather")
	{
		var type = GetEntityResourceType(data.target);
		if (type === undefined)
			return;
		else if (type.specific == "fruit")
			g_TriggerData.action_UnitOrderedToGatherFruit(data);
		else if (type.specific == "tree")
			g_TriggerData.action_UnitOrderedToGatherTree(data);
		else if (type.specific == "rock")
			g_TriggerData.action_UnitOrderedToGatherStoneQuarry(data);
		else if (type.specific == "ore")
			g_TriggerData.action_UnitOrderedToGatherMetalMine(data);
		else if (type.generic == "treasure")
			g_TriggerData.action_UnitOrderedToGatherTreasure(data);
	}
}

g_TriggerData.action_HandleMultipleEntityOrders = function(data)
{
	if (data.order == "Order.Walk")
	{
		if (!g_TriggerData.movingTutorial2Completed)
		{
			g_TriggerData.movingTutorial2Completed = true;
			g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_MovingAGroupOfUnitsTutorial");
			PushGUINotificationInTopCenter(1, "Very Good.");
			g_TriggerData.cmpTrigger.StopUpdateTimer(); // Stop the "Always" timer.
			g_TriggerData.cmpTrigger.DoAfterDelay(6000, "action_IntroductionToResources", {});
		}
	}
}

// This one is called when the player moves a unit.
g_TriggerData.action_PlayerMovedAUnit = function(data)
{
	if (!g_TriggerData.movingTutorial1Completed)
	{
		g_TriggerData.movingTutorial1Completed = true;
		PushGUINotificationInTopCenter(1, "Good. You can also select a group of units by holding the left mouse button and dragging it. Select a group of units and order them to move.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_MovingUnitsTutorial");
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_MovingAGroupOfUnitsTutorial");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(10000, 10000);
	}
}

// This one is called when the player orders a unit to gather from fruit bushes
g_TriggerData.action_UnitOrderedToGatherFruit = function(data)
{
	if (!g_TriggerData.gatheringFruitComepleted)
	{
		PushGUINotificationInTopCenter(1, "Excelent. Now your units will move and gather food from those bushes.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnGatheringFood");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(5000, 10000);
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnGatheringWood");
		
		g_TriggerData.gatheringFruitComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from trees
g_TriggerData.action_UnitOrderedToGatherTree = function(data)
{
	if (!g_TriggerData.gatheringTreeComepleted)
	{
		PushGUINotificationInTopCenter(1, "Perfect. Now your units will move and gather wood from the trees.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnGatheringWood");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(5000, 10000);
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnGatheringStone");
		
		g_TriggerData.gatheringTreeComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from stone quarries
g_TriggerData.action_UnitOrderedToGatherStoneQuarry = function(data)
{
	if (!g_TriggerData.gatheringStoneComepleted)
	{
		PushGUINotificationInTopCenter(1, "Good. Now your units will move and gather stone from the quarry.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnGatheringStone");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(5000, 10000);
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnGatheringMetal");
		
		g_TriggerData.gatheringStoneComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from metal mines
g_TriggerData.action_UnitOrderedToGatherMetalMine = function(data)
{
	if (!g_TriggerData.gatheringMetalComepleted)
	{
		PushGUINotificationInTopCenter(1, "Very Good. Now your units will move and gather metal from the ore.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnGatheringMetal");
		
		// Spawn a cavalry unit for the player in his/her civic center.
		var playerEntities = GetEntitiesByPlayer(1); // Get all of the player's entities
		for each (entity in playerEntities)
		{
			if (EntityHasClass(entity, "CivilCentre"))
			{
				BuildingSpawnUnits(entity, "skirmish/units/default_cavalry", 1);
				break;
			}
		}
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(5000, 10000);
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnExploration");
		g_TriggerData.cmpTrigger.EnableRangeTrigger("action_StartTreasureTutorial");
		
		g_TriggerData.gatheringMetalComepleted = true;
	}
}

// This one is called when a unit gets near the market in the south-east
g_TriggerData.action_StartTreasureTutorial = function(data)
{
		PushGUINotificationInTopCenter(1, "You have found some treasures. Treasures give large ammount of their representing resources instantly. Order your rider to gather the treasures.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnExploration");
		g_TriggerData.cmpTrigger.DisableRangeTrigger("action_StartTreasureTutorial");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		g_TriggerData.cmpTrigger.StartUpdateTimer(5000, 10000);
		
		g_TriggerData.cmpTrigger.EnableTrigger("Always", "action_TutorialOnGatheringTreasure");
}

// This one is called when the player orders a unit to gather from treasures
g_TriggerData.action_UnitOrderedToGatherTreasure = function(data)
{
	if (!g_TriggerData.gatheringTreasureComepleted)
	{
		PushGUINotificationInTopCenter(1, "Outstanding. This tutorial is now done.");
		g_TriggerData.cmpTrigger.DisableTrigger("Always", "action_TutorialOnGatheringTreasure");
		
		g_TriggerData.cmpTrigger.StopUpdateTimer();   // Stop the "Always" timer

		g_TriggerData.gatheringTreasureComepleted = true;
		
		g_TriggerData.cmpTrigger.DoAfterDelay(4000, "action_Victory", {});
	}
}

g_TriggerData.action_Victory = function(data)
{
	SetPlayerWon(1);
}

///////////////////////////////////////////////////////////////////////////////
//                              Setting Actions Up                           //
///////////////////////////////////////////////////////////////////////////////

// The set of global trigger actions. The name must always be "g_TriggerData"

// Make the trigger actions global so that we can use it in the trigger component.
Engine.RegisterGlobal("g_TriggerData", g_TriggerData);

///////////////////////////////////////////////////////////////////////////////
//                            Startup Actions                                //
///////////////////////////////////////////////////////////////////////////////

g_TriggerData.cmpTrigger.DoAfterDelay(2000, "action_WelcomeMessage", {});

///////////////////////////////////////////////////////////////////////////////
//                           Register Triggers                               //
///////////////////////////////////////////////////////////////////////////////

g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_MovingUnitsTutorial", false); // "false" means that it is disabled by default
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_MovingAGroupOfUnitsTutorial", false);
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnGatheringFood", false);
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnGatheringWood", false);
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnGatheringStone", false);
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnGatheringMetal", false);
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnExploration", false);
g_TriggerData.cmpTrigger.RegisterOnUnitRangeFromEntityTrigger(
	"entered", 
	226, 
	40, 
	"action_StartTreasureTutorial", 
	[1], 
	0, //To get better performance, it is adviced to use IID_DamageReceiver instead of 0 although it would limit the entites that signal the event to the ones that can recieve damage.
	false
); // This special trigger calls the action when an entity owned by player 1 enters a circular area of 40 game meters centered around entity 226 (the gaia market). 
g_TriggerData.cmpTrigger.RegisterTrigger("Always", "action_TutorialOnGatheringTreasure", false);
g_TriggerData.cmpTrigger.RegisterTrigger("OnUnitIssuedOrder", "action_HandleOrders", true);
g_TriggerData.cmpTrigger.RegisterTrigger("OnUnitsIssuedOrder", "action_HandleMultipleEntityOrders", true);
