// Global variables
var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);

var movingTutorial1Completed = false;
var movingTutorial2Completed = false;
var gatheringFruitComepleted = false;
var gatheringTreeComepleted = false;
var gatheringStoneComepleted = false;
var gatheringMetalComepleted = false;
var gatheringTreasureComepleted = false;
var movingTutorial2EntitiesOrdered = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                       Function defenitions                                                       //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// These actions control the tutorials in which the player has no interactin with.
function action_WelcomeMessage(data)
{
	PushGUINotificationInTopCenter(1, "Welcome to the basic tutorial of the 0 A.D.!");
	cmpTrigger.DoAfterDelay(3000, "action_IntroductionMessage", {});
}

function action_IntroductionMessage(data)
{
	PushGUINotificationInTopCenter(1, "In this tutorial you will learn the basics of the gameplay of 0 A.D.");
	cmpTrigger.DoAfterDelay(6000, "action_CameraMovementTutorial1", {});
}

function action_CameraMovementTutorial1(data)
{
	PushGUINotificationInTopCenter(1, "First, we try the camera movements. To move your camera, move the mouse pointer to the edge of the screen.");
	cmpTrigger.DoAfterDelay(8000, "action_CameraMovementTutorial2", {});
}

function action_CameraMovementTutorial2(data)
{
	PushGUINotificationInTopCenter(1, "You can also move the camera using W,A,S,D or arrow keys on the keyboard, or by holding the middle mouse button and moving it.");
	cmpTrigger.DoAfterDelay(15000, "action_CameraMovementTutorial3", {});
}

function action_CameraMovementTutorial3(data)
{
	PushGUINotificationInTopCenter(1, "To rotate the camera, hold Ctrl and one of the arrow keys.");
	cmpTrigger.DoAfterDelay(7000, "action_IntroductionToUnits", {});
}

function action_IntroductionToUnits(data)
{
	PushGUINotificationInTopCenter(1, "Now it is time to work with units. Move the mouse pointer on one of the units present in the screen, and left-click to select it. Then, right-click on an open part of the terrain.");
	cmpTrigger.EnableTrigger("trigger_MovingUnitsTutorial");
	cmpTrigger.StartUpdateTimer(12000, 10000); // Sets the interval in which the event "Always" occurs to 12000ms (12 seconds). Used to remind the players of what they should do.
}

function action_IntroductionToResources(data)
{
	PushGUINotificationInTopCenter(1, "Resource gathering play a central role in 0 A.D.. There are four resources found in the game. Food, Wood, Stone and metal. You can find your current stockpile in the upper left section of the screen.");
	cmpTrigger.DoAfterDelay(15000, "action_MoreDescriptionAboutResources", {});
}

function action_MoreDescriptionAboutResources(data)
{
	PushGUINotificationInTopCenter(1, "Resources are required to train units, build structures and research technologies.");
	cmpTrigger.DoAfterDelay(4000, "action_EnablingGatheringFoodTrigger", {});
}

function action_EnablingGatheringFoodTrigger(data)
{
	cmpTrigger.StartUpdateTimer(10000, 1000); // Sets the interval in which the event "Always" occurs to 10000ms (10 seconds). Used to remind the players of what they should do.
	cmpTrigger.EnableTrigger("trigger_TutorialOnGatheringFood");
}

// These actions remind the players of what they should do.
function action_MovingUnitsTutorial(data)
{
	PushGUINotificationInTopCenter(1, "Select a unit by left clicking on it, and then, right-click on an open part of the terrain.");
}

function action_MovingAGroupOfUnitsTutorial(data)
{
	PushGUINotificationInTopCenter(1, "Hold the left button of the mouse and drag it to select a group of units. Then, order them to move.");
}

function action_TutorialOnGatheringFood(data)
{
	PushGUINotificationInTopCenter(1, "Food is gathered from farms, fruit trees and bushes, animals, fishes and farms. Select some of your units and right-click on the berry bushes in the east to gather food.");
}

function action_TutorialOnGatheringWood(data)
{
	PushGUINotificationInTopCenter(1, "Wood is gathered from trees. Select some of your units and right-click on a tree to gather wood.");
}

function action_TutorialOnGatheringStone(data)
{
	PushGUINotificationInTopCenter(1, "Stone is gathered from stone quarries. Select some of your units and right-click on the quarry in the south of your town to gather stone.");
}

function action_TutorialOnGatheringMetal(data)
{
	PushGUINotificationInTopCenter(1, "Metal is gathered from metal ores. Select some of your units and right-click on the rock in the north-east of your town to gather metal.");
}

function action_TutorialOnExploration(data)
{
	PushGUINotificationInTopCenter(1, "Notice that you can only see the immidiate surroundings of you town. To see further, select your rider and order him to move along the south-eastern road into the black area.");
}

function action_TutorialOnGatheringTreasure(data)
{
	PushGUINotificationInTopCenter(1, "Order your rider to gather the treasures. To do so, select your rider and right-click on one of the treasures near the road.");
}

// Thse actions set the next stage of the tutorial after the player has successfully done the previous ones

function action_HandleOrders(data)
{
	if (data.order == "Order.Walk")
		action_PlayerMovedAUnit(data);
	else if (data.order == "Order.Gather")
	{
		var type = GetEntityResourceType(data.target);
		if (type === undefined)
			return;
		else if (type.specific == "fruit")
			action_UnitOrderedToGatherFruit(data);
		else if (type.specific == "tree")
			action_UnitOrderedToGatherTree(data);
		else if (type.specific == "rock")
			action_UnitOrderedToGatherStoneQuarry(data);
		else if (type.specific == "ore")
			action_UnitOrderedToGatherMetalMine(data);
		else if (type.generic == "treasure")
			action_UnitOrderedToGatherTreasure(data);
	}
}

// This one is called when the player moves a unit.
function action_PlayerMovedAUnit(data)
{
	if (!movingTutorial1Completed)
	{
		movingTutorial1Completed = true;
		PushGUINotificationInTopCenter(1, "Good. You can also select a group of units by holding the left mouse button and dragging it. Select a group of units and order them to move.");
		cmpTrigger.DisableTrigger("trigger_MovingUnitsTutorial");
		
		cmpTrigger.EnableTrigger("trigger_MovingAGroupOfUnitsTutorial");
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 10000);
	}
	else if (!movingTutorial2Completed)
	{
		++movingTutorial2EntitiesOrdered;
		if (movingTutorial2EntitiesOrdered >= 2)
		{
			movingTutorial2Completed = true;
			cmpTrigger.DisableTrigger("trigger_MovingAGroupOfUnitsTutorial");
			PushGUINotificationInTopCenter(1, "Very Good.");
			cmpTrigger.StopUpdateTimer(); // Stop the "Always" timer.
			cmpTrigger.DoAfterDelay(6000, "action_IntroductionToResources", {});
		}
	}
}

// This one is called when the player orders a unit to gather from fruit bushes
function action_UnitOrderedToGatherFruit(data)
{
	if (!gatheringFruitComepleted)
	{
		PushGUINotificationInTopCenter(1, "Excelent. Now your units will move and gather food from those bushes.");
		cmpTrigger.DisableTrigger("trigger_TutorialOnGatheringFood");
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 5000);
		
		cmpTrigger.EnableTrigger("trigger_TutorialOnGatheringWood");
		
		gatheringFruitComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from trees
function action_UnitOrderedToGatherTree(data)
{
	if (!gatheringTreeComepleted)
	{
		PushGUINotificationInTopCenter(1, "Perfect. Now your units will move and gather wood from the trees.");
		cmpTrigger.DisableTrigger("trigger_TutorialOnGatheringWood");
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 5000);
		
		cmpTrigger.EnableTrigger("trigger_TutorialOnGatheringStone");
		
		gatheringTreeComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from stone quarries
function action_UnitOrderedToGatherStoneQuarry(data)
{
	if (!gatheringStoneComepleted)
	{
		PushGUINotificationInTopCenter(1, "Good. Now your units will move and gather stone from the quarry.");
		cmpTrigger.DisableTrigger("trigger_TutorialOnGatheringStone");
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 5000);
		
		cmpTrigger.EnableTrigger("trigger_TutorialOnGatheringMetal");
		
		gatheringStoneComepleted = true;
	}
}

// This one is called when the player orders a unit to gather from metal mines
function action_UnitOrderedToGatherMetalMine(data)
{
	if (!gatheringMetalComepleted)
	{
		PushGUINotificationInTopCenter(1, "Very Good. Now your units will move and gather metal from the ore.");
		cmpTrigger.DisableTrigger("trigger_TutorialOnGatheringMetal");
		
		// Spawn a cavalry unit for the player in his/her civic center.
		var playerEntities = GetEntitiesByPlayer(1); // Get all of the player's entities
		for each (entity in playerEntities)
		{
			if (EntityHasClass(entity, "CivilCentre"))
			{
				BuildingSpawnUnits(entity, "skirmish/units/default_cavalry", 1);
				break;
			}
		}
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 5000);
		
		cmpTrigger.EnableTrigger("trigger_TutorialOnExploration");
		cmpTrigger.EnableTrigger("trigger_FindingTheMarket");
		
		gatheringMetalComepleted = true;
	}
}

// This one is called when a unit gets near the market in the south-east
function action_StartTreasureTutorial(data)
{
		PushGUINotificationInTopCenter(1, "You have found some treasures. Treasures give large ammount of their representing resources instantly. Order your rider to gather the treasures.");
		cmpTrigger.DisableTrigger("trigger_FindingTheMarket");
		cmpTrigger.DisableTrigger("trigger_TutorialOnExploration");
		
		cmpTrigger.StopUpdateTimer();   // Reset the "Always" timer
		cmpTrigger.StartUpdateTimer(10000, 5000);
		
		cmpTrigger.EnableTrigger("trigger_TutorialTreasures");
}

// This one is called when the player orders a unit to gather from treasures
function action_UnitOrderedToGatherTreasure(data)
{
	if (!gatheringTreasureComepleted)
	{
		PushGUINotificationInTopCenter(1, "Outstanding. This tutorial is now done.");
		cmpTrigger.DisableTrigger("trigger_TutorialTreasures");
		
		cmpTrigger.StopUpdateTimer();   // Stop the "Always" timer

		gatheringTreasureComepleted = true;
		
		cmpTrigger.DoAfterDelay(4000, "action_Victory", {});
	}
}

function action_Victory(data)
{
	SetPlayerWon(1);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                            Setting Actions Up                                                    //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The set of global trigger actions. The name must always be "g_TriggerActions"
var g_TriggerActions = {
	"action_WelcomeMessage": action_WelcomeMessage,
	"action_IntroductionMessage": action_IntroductionMessage,
	"action_CameraMovementTutorial1": action_CameraMovementTutorial1,
	"action_CameraMovementTutorial2": action_CameraMovementTutorial2,
	"action_CameraMovementTutorial3": action_CameraMovementTutorial3,
	"action_IntroductionToUnits": action_IntroductionToUnits,
	"action_MovingUnitsTutorial": action_MovingUnitsTutorial,
	"action_MovingAGroupOfUnitsTutorial": action_MovingAGroupOfUnitsTutorial,
	"action_HandleOrders": action_HandleOrders,
	"action_IntroductionToResources": action_IntroductionToResources,
	"action_MoreDescriptionAboutResources": action_MoreDescriptionAboutResources,
	"action_EnablingGatheringFoodTrigger": action_EnablingGatheringFoodTrigger,
	"action_TutorialOnGatheringFood": action_TutorialOnGatheringFood,
	"action_TutorialOnGatheringWood": action_TutorialOnGatheringWood,
	"action_TutorialOnGatheringStone": action_TutorialOnGatheringStone,
	"action_TutorialOnGatheringMetal": action_TutorialOnGatheringMetal,
	"action_TutorialOnExploration": action_TutorialOnExploration,
	"action_StartTreasureTutorial": action_StartTreasureTutorial,
	"action_Victory": action_Victory
};

// Make the trigger actions global so that we can use it in the trigger component.
Engine.RegisterGlobal("g_TriggerActions", g_TriggerActions);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                            Startup Actions                                                       //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

cmpTrigger.DoAfterDelay(2000, "action_WelcomeMessage", {});

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                            Register Triggers                                                     //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

cmpTrigger.RegisterTrigger("trigger_MovingUnitsTutorial", "Always", "action_MovingUnitsTutorial", false); // "false" means that it is disabled by default
cmpTrigger.RegisterTrigger("trigger_MovingAGroupOfUnitsTutorial", "Always", "action_MovingAGroupOfUnitsTutorial", false);
cmpTrigger.RegisterTrigger("trigger_TutorialOnGatheringFood", "Always", "action_TutorialOnGatheringFood", false);
cmpTrigger.RegisterTrigger("trigger_TutorialOnGatheringWood", "Always", "action_TutorialOnGatheringWood", false);
cmpTrigger.RegisterTrigger("trigger_TutorialOnGatheringStone", "Always", "action_TutorialOnGatheringStone", false);
cmpTrigger.RegisterTrigger("trigger_TutorialOnGatheringMetal", "Always", "action_TutorialOnGatheringMetal", false);
cmpTrigger.RegisterTrigger("trigger_TutorialOnExploration", "Always", "action_TutorialOnExploration", false);
cmpTrigger.RegisterOnUnitRangeFromEntityTrigger("entered", "trigger_FindingTheMarket", 226, 40, "action_StartTreasureTutorial", [1], IID_DamageReceiver, false); // This special trigger calls the action when an entity owned by player 1 enters a circular area of 40 game meters centered around entity 226 (the gaia market).
cmpTrigger.RegisterTrigger("trigger_TutorialTreasures", "Always", "action_TutorialOnGatheringTreasure", false);
cmpTrigger.RegisterTrigger("trigger_UnitIssuedOrder", "OnUnitIssuedOrder", "action_HandleOrders");